const path = require('path')
const HTMLWebpackPlugin = require('html-webpack-plugin')
const open = require('opn')   //打开浏览器
const chalk = require('chalk')   //改变命令行中输出日志颜色
const ip = require('ip').address()    //获取本机IP地址
const CleanWebpackPlugin = require('clean-webpack-plugin');
const copyWebpackPlugin = require('copy-webpack-plugin');

module.exports = {
	//配置入口文件
	entry: path.resolve(__dirname, 'src/index.ts'),    //单入口字符串类型、多入口数组格式、多入口对象形式
	//配置输出文件
	output: {
		//path 生成文件的根目录、需要传入一个绝对路径
		//publicPath 生成资源文件的共用路径 例如生成css js img字体图标
		//filname 定义输出文件名规则
			// -> name 输出文件的name 如果entry的字段值是对象 那么此name对应entry字段值的key
			// -> hash 编译时生成一个hash值 可以设置hash值的长度 [hash: n]中的n就为hash的长度值
			// -> chunkhash 指向当前chunk的一个hash版本，即在同一次编译中，每个chunk的hash都是不一样的，而在两次编译中，如果某个chunk根本没有发生变化，那么该chunk的hash也就不会发生变化，这在缓存的层面上来说，就是把缓存的颗粒度精细到具体某个chunk，只要chunk不变，该chunk的浏览器缓存就可以继续使用
			// -> chunkFilename 与filename类似，都是用来定义生成文件的命名方式，只不过，chunkFilename参数指定的是除入口文件外的chunk(这些chunk通常是由于webpack对代码的优化所形成的，比如因实际行动的情况来异步加载)的命名
		path: path.resolve(__dirname, 'dist'),
		filename: 'index.js',
		publicPath: ''
	},
	devtool: 'eval-source-map',
	//webpack4.X新增环境配置项
	mode: 'development',    //mode是webpack4.X新增的配置，可以在此处声明，也可以在命令中 --mode development  取值有三个：development production none
	module: {
		rules: [
			//处理ts文件
			{
				test: /\.ts?$/,
				use: 'ts-loader',
				exclude: /node_modules/
			},
			{
				test: /\.js$/,
				use: 'babel-loader'
			}
		]
	},
	//插件配置项
	plugins: [
		new HTMLWebpackPlugin({
			filename: 'index.html',    //必填 输出文件的名称
			template: path.resolve(__dirname, 'index.html'),   //必填 模板文件的路径
			title: 'pdfToImg'   //配置生成页面的标题
		}),
		new CleanWebpackPlugin(),
		//打包静态资源
		new copyWebpackPlugin([
			{
				from:path.resolve(__dirname+'/static'),// 打包的静态资源目录地址
				to:'./static' // 打包到dist下面的static
			}
		]),
	],
	//开发服务配置项
	devServer: {
		//color console中打印彩色日志
		//historyApiFallback 任意的404响应都被替代为index.html。启用该配置后，项目中任何找不到的链接都会被强制链接到index.html页面
		//host 指定一个host 默认是localhost 如果你希望服务器外部访问，指定如下：host: "0.0.0.0",如果用手机通过IP访问，则可设置为本机局域网IP
		//hot 启用webpack的模块热替换特性
		//port 配置端口号 默认是8080
		//progress 将编译进度输出到控制台
		//overlay 在浏览页面输出报错信息
		port: '8066',   //端口号
		contentBase: path.resolve(__dirname, 'dist'),
		historyApiFallback: true,
		host: ip,
		overlay: true,
		after() {
			open(`http://${ip}:${this.port}`)
				.then(() => {
					console.log(chalk.cyan(`成功打开链接：http://${ip}:${this.port}`))
				}).catch((err) => {
					console.log(chalk.red(err))
			})
		}
	}

}